# <><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>
# AUTHOR:       Philippe Massicotte
#
# DESCRIPTION:
#
# Fit the equation 2 from Frey's geometric model on the 60 COPS profils taken
# under high snow condition.
#
# Frey, K. E., Perovich, D. K., & Light, B. (2011). The spatial distribution of
# solar radiation under a melting Arctic sea ice cover. Geophysical Research
# Letters, 38(22), n/a-n/a. https://doi.org/10.1029/2011GL049421
# <><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>

rm(list = ls())

cops <- read_csv("data/clean/cops.csv")

cops <- cops %>% 
  filter(hole_type == "H")

## 60 pofils on high snow (H). These present all the same pattern.
cops %>% 
  group_by(profile_filename) %>% 
  nest()

p <- cops %>% 
  drop_na(par_d_p24h_ein_m_2_day_1) %>% 
  ggplot(aes(x = par_d_p24h_ein_m_2_day_1, y = depth)) +
  geom_path() +
  scale_y_reverse() +
  facet_wrap(~profile_filename, scales = "free")

rss <- function(par, data) {
  
  y <- data$par_d_p24h_ein_m_2_day_1
  x <- data$depth
  
  p <- par[1]
  r <- par[2]
  n <- par[3]
  i <- par[4]
  k <- par[5]
  
  yy <- calculate_light(par, x)
  
  rss <- sum((y - yy)^2, na.rm = TRUE)
  
  return(rss)
  
}

calculate_light <- function(par, x) {
  
  p <- par[1]
  r <- par[2]
  n <- par[3]
  i <- par[4]
  k <- par[5]
  
  yy <- pi * i * (1 + p * (n - 1) * cos(atan(r / x)) ^ 2) * exp(-k * x)
  
  return(yy)
  
}

fit_model <- function(df) {
  
  mod <- optim(par = c(
    p = 0.1,
    r = 4,
    n = 11,
    i = 0.008,
    k = 0.06),
    rss,
    data = df)
  
  return(mod)
  
}

## Predict light using the model generated by "optim".
predict_light <- function(data, model) {
  
  pred <- calculate_light(model$par, data$depth)
  # pred <- data_frame(depth = data$depth, pred = pred)
  return(pred)
}

res <- cops %>% 
  drop_na(par_d_p24h_ein_m_2_day_1) %>% 
  group_by(profile_filename, posixct_date_utc, hole_type) %>% 
  nest() %>% 
  mutate(model = map(data, fit_model)) %>% 
  mutate(pred = map2(data, model, predict_light))

p <- res %>% 
  unnest(data, pred) %>% 
  ggplot(aes(x = par_d_p24h_ein_m_2_day_1, y = depth)) +
  geom_point(aes(color = "Observed")) +
  geom_path(aes(x = pred, color = "Modeled")) +
  scale_y_reverse() +
  facet_wrap(~profile_filename, scales = "free") +
  labs(title = "RAW and modeled PAR profils") +
  theme(legend.title = element_blank()) +
  theme(legend.position = c(0.95, 0.05), legend.justification = c(1, 0)) +
  scale_color_manual(values = c("Observed" = "black", "Modeled" = "red"))

ggsave("graphs/fitted_profiles.pdf", height = 14, width = 18)
embed_fonts("graphs/fitted_profiles.pdf")

params <- res %>%
  mutate(params = map(model, broom::tidy)) %>% 
  unnest(params) 

params %>% 
  ggplot(aes(x = value)) +
  geom_histogram() +
  facet_wrap(~parameter, scales = "free")

## save model results for later use
write_csv(params, "data/clean/high_snow_geometric_model_parameters.csv")


# Parameter correlations --------------------------------------------------

params <- params %>% 
  spread(parameter, value)

pdf("graphs/parameters_scatterplot_matrix.pdf")
languageR::pairscor.fnc(select(params, i:r))
dev.off()

pdf("graphs/parameters_pca.pdf")
biplot(prcomp(select(params, i:r), center = TRUE, scale. = TRUE))
dev.off()
